<?php

namespace Drupal\pair_store\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;

class PairStoreController extends ControllerBase {

  protected $configFactory;

  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  public function list() {
    $config = $this->configFactory->get('pair_store.settings');
    $pairs = $config->get('pairs') ?? [];

    $header = [
      $this->t('Key'),
      $this->t('Value'),
      $this->t('Operations'),
    ];

    $rows = [];

    foreach ($pairs as $key => $value) {
      $operations = [
        'data' => [
          '#type' => 'operations',
          '#links' => [
            'edit' => [
              'title' => $this->t('Edit'),
              'url' => Url::fromRoute('pair_store.edit', ['key' => $key]),
            ],
            'delete' => [
              'title' => $this->t('Delete'),
              'url' => Url::fromRoute('pair_store.delete', ['key' => $key]),
            ],
          ],
        ],
      ];

      $rows[] = [
        $key,
        $value,
        $operations,
      ];
    }

    return [
      '#type' => 'container',
      'create_link' => [
        '#type' => 'link',
        '#title' => $this->t('Create New Pair'),
        '#url' => Url::fromRoute('pair_store.form'),
        '#attributes' => ['class' => ['button', 'button--primary']],
      ],
      'pairs_table' => [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#empty' => $this->t('No pairs found.'),
        '#attributes' => ['class' => ['pair-store-table']],
      ],
    ];
  }

  /**
   * Edits a key-value pair.
   */
  public function edit(Request $request, $key): RedirectResponse|array {
    $config = $this->config('pair_store.settings');
    $pairs = $config->get('pairs') ?? [];

    if (!isset($pairs[$key])) {
      $this->messenger()->addError($this->t('Key not found.'));
      return $this->redirect('pair_store.list');
    }

    $value = $pairs[$key];

    return \Drupal::formBuilder()->getForm('Drupal\pair_store\Form\PairEditForm', $key, $value);
  }

  /**
   * Deletes a key-value pair.
   */
  public function delete($key): RedirectResponse {
    $config = $this->configFactory->getEditable('pair_store.settings');
    $pairs = $config->get('pairs') ?? [];

    if (isset($pairs[$key])) {
      unset($pairs[$key]);
      $config->set('pairs', $pairs)->save();
      $this->messenger()->addStatus($this->t('Key "@key" deleted.', ['@key' => $key]));
    } else {
      $this->messenger()->addWarning($this->t('Key not found.'));
    }

    return $this->redirect('pair_store.list');
  }

}
