<?php

namespace Drupal\pair_store\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

class PairEditForm extends FormBase {

  protected $key;
  protected $value;

  public function __construct($key = '', $value = '') {
    $this->key = $key;
    $this->value = $value;
  }

  public function getFormId() {
    return 'pair_edit_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state, $key = '', $value = '') {
    $this->key = $key;
    $this->value = $value;

    $form['key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Key'),
      '#default_value' => $key,
      '#disabled' => TRUE,
    ];

    $form['value'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Value'),
      '#default_value' => $value,
      '#required' => TRUE,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    return $form;
  }

  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $value = $form_state->getValue('value');

    $config = \Drupal::configFactory()->getEditable('pair_store.settings');
    $pairs = $config->get('pairs') ?? [];

    $pairs[$this->key] = $value;
    $config->set('pairs', $pairs)->save();

    $this->messenger()->addStatus($this->t('Value updated for "@key".', ['@key' => $this->key]));

    $form_state->setRedirect('pair_store.list');
  }
}
