<?php

namespace Drupal\pair_store\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

class PairStoreForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['pair_store.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'pair_store_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('pair_store.settings');
    $saved_pairs = $config->get('pairs') ?? [];

    // Determine how many rows to show.
    $num_rows = $form_state->get('num_rows');
    if ($num_rows === NULL) {
      $num_rows = count($saved_pairs) ?: 1;
      $form_state->set('num_rows', $num_rows);
    }

    $form['#prefix'] = '<div id="pair-form-wrapper">';
    $form['#suffix'] = '</div>';

    $form['pairs'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Key-Value Pairs'),
      '#tree' => TRUE,
    ];

    // Build each pair input row.
    for ($i = 0; $i < $num_rows; $i++) {
      $key = array_keys($saved_pairs)[$i] ?? '';
      $value = $saved_pairs[$key] ?? '';

      $form['pairs'][$i]['key'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Key'),
        '#default_value' => $key,
        '#size' => 20,
        '#required' => FALSE,
      ];

      $form['pairs'][$i]['value'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Value'),
        '#default_value' => $value,
        '#size' => 40,
        '#required' => FALSE,
      ];
    }

    $form['add_pair'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add another pair'),
      '#submit' => ['::addOne'],
      '#ajax' => [
        'callback' => '::ajaxCallback',
        'wrapper' => 'pair-form-wrapper',
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * AJAX callback to rebuild the form when adding a pair.
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state) {
    return $form;
  }

  /**
   * Submit handler for the "Add another pair" button.
   */
  public function addOne(array &$form, FormStateInterface $form_state): void {
    $num_rows = $form_state->get('num_rows');
    $form_state->set('num_rows', $num_rows + 1);
    $form_state->setRebuild(TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $input = $form_state->getValue('pairs');
    $pairs = [];

    foreach ($input as $row) {
      $key = trim($row['key']);
      $value = trim($row['value']);
      if ($key !== '') {
        $pairs[$key] = $value;
      }
    }

    $this->config('pair_store.settings')
      ->set('pairs', $pairs)
      ->save();

    $this->messenger()->addStatus($this->t('Key-value pairs saved successfully.'));
    parent::submitForm($form, $form_state);
  }

}
